;*************************************************************************
;*									 *
;*				AVR universal Bootloader		 *
;*									 *
;*			Author: Peter Dannegger				 *
;*									 *
;*************************************************************************
.nolist
.include "compat.h"
.include "fastload.h"
.listmac
.list
	.org	0x0000
.if FlashEnd > 0xFFF
	jmp	init
.else
	rjmp	init
.endif
	.org	BootStart
init:
	cli
	ldi	a0, low (RamEnd)
	out	SPL, a0
.ifdef SPH
	ldi	a0, high(RamEnd)
	out	SPH, a0
.endif
	clr	zerol
	clr	zeroh
	sbi	SRX_PORT, SRX
	sbi	STX_PORT, STX
	sbi	STX_DDR, STX
.if CRC > 0
	ldi	polynoml, 0x01
	ldi	polynomh, 0xA0
.endif
;-------------------------------------------------------------------------
.include "abaud.inc"
.include "password.inc"
;-------------------------------------------------------------------------
;				Receive commands
;-------------------------------------------------------------------------
;00	get bootloader revision
;01	get buffer size
;02	get target signature
;03	get user flash size
;04	program flash
;05	start application
;06	check crc
;07	verify flash
;-------------------------------------------------------------------------
main_ok:
	ldi	a0, SUCCESS
_cex1:
	rcall	putchar
_cex2:
	rcall	getchar
	cpi	a0, COMMAND
	brne	_cex2
	rcall	getchar
	cpi	a0, 4
	brcs	sendmessage		; 0 ... 3
	breq	program			; 4
	cpi	a0, 6
	brcc	_cex3
timeout:				; 5
	out	STX_PORT, zerol
	out	STX_DDR, zerol
	out	SRX_PORT, zerol
.ifdef BootFuse
.if FlashEnd > 0x0FFF
	jmp	0x0000
.else
	rjmp	0x0000			; run application
.endif
.else
	rjmp	BootStart-1		; run user application
.endif
_cex3:
;-------------------------------------------------------------------------
.if CRC > 0
	brne	_cex4
.include "checkcrc.inc"			; 6
_cex4:
.endif
;-------------------------------------------------------------------------
.if VERIFY > 0
	cpi	a0, 7
	brne	_cex5
.include "verify.inc"			; 7
_cex5:
.endif
;-------------------------------------------------------------------------
	ldi	a0, BADCOMMAND
	rjmp	_cex1
main_error:
	ldi	a0, FAIL
	rjmp	_cex1
;-------------------------------------------------------------------------
SendMessage:
.include "message.inc"
;-------------------------------------------------------------------------
.ifdef BootFuse
.include "progmega.inc"		; mega with bootstart fuse set
.else
.include "progtiny.inc"		; tiny, mega without RWW section
.endif
;-------------------------------------------------------------------------
.if CRC > 0
.include "uartcrc.inc"
.else
.include "uart.inc"
.endif
;-------------------------------------------------------------------------
Password:
	.db	"Peda", 0, 0		; 'a' was recognized by ABAUD
.list
Messages:
	.db \
	3, high(Version), low(Version), \
	3, high(2*BufferSize), low(2*BufferSize), \
	4, SIGNATURE_000, SIGNATURE_001, SIGNATURE_002, \
	4, byte3(UserFlash), byte2(UserFlash), byte1(UserFlash)
;-------------------------------------------------------------------------
